//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  bistcontroller.v
//  Revision        :  $Revision: 1.0 $
//  Checkin Date    :  $Date: 2008/05/28 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <bistcontroller>
//  Description     :  <the bist controller>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************

module bistcontroller(/*AUTOARG*/
  // Outputs
  bist_done, bist_en, bist_lfsr_en, bist_misr_en, bist_scan_en,
  bist_weighting_sel, bist_flipping_sel, bist_delaytest, bist_scan_conf_out,
  // Inputs
  bist_start, bist_clk, bist_rst_n, bist_scan_conf_in, bist_scan_conf_en
  );

   `include "chip_params.v"
   `include "bistcontroller_params.v"

   //outputs
   output                       bist_done;
   output                       bist_en;
   output                       bist_lfsr_en;
   output                       bist_misr_en;
   output                       bist_scan_en;
   output [WEIGHTING_SEL-1:0]   bist_weighting_sel;
   output [NUM_SCAN_CHAINS-1:0] bist_flipping_sel;
   output 			bist_delaytest;
   output 			bist_scan_conf_out;
   
   
   //inputs
   input 			bist_start;
   input 			bist_clk;
   input 			bist_rst_n;
   input 			bist_scan_conf_in;
   input 			bist_scan_conf_en;
   

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   reg [NUM_SCAN_CHAINS-1:0] bist_flipping_sel;
   reg [WEIGHTING_SEL-1:0] bist_weighting_sel;
   // End of automatics
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire                 bist_flip_en;           // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 bist_flip_load;         // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 bist_weight_load;       // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire [WEIGHTING_SEL-1:0] bist_weight_sel;    // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 bitcounter_inc;         // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 bitcounter_rst;         // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 misr_enable_set;        // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 patterncounter_inc;     // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 patterncounter_rst;     // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 testcounter_inc;        // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   wire                 testcounter_rst;        // From U_BISTCONTROLLER_FSM of bistcontroller_fsm.v
   // End of automatics

   reg [SHIFTCOUNTER_BITS-1:0] 	 bit_counter_r;     //length of scan chain+1 (can count longer)
   reg [SHIFTCOUNTER_BITS-1:0] 	 bits_to_apply_r;    //number of bits in a pattern to apply
   reg [PATTERNCOUNTER_BITS-1:0] pattern_counter_r; //counts number of patterns applied
   reg [PATTERNCOUNTER_BITS-1:0] pattern_to_apply_r; //number of patter to apply 
   reg [CONFCOUNTERSIZE-1:0] 	 test_counter_r;     //"state" of bist-tests cycle
   reg [CONFCOUNTER-1:0] 	 conf_r;             //0/1 configuration for apply/not apply test
   reg [NUM_SCAN_CHAINS-1:0] 	 flipping_conf_r;
   reg 				 misr_enable_r;
				 
   //config scan chain: bist_load
   //input -> pattern_to_apply_r , bits_to_apply_r , conf_r , flipping_conf_r -> output
   assign 			 bist_scan_conf_out = flipping_conf_r[NUM_SCAN_CHAINS-1];
   
   
   /*
    * misr enable 
    * do not run misr during first scan, prevent X into misr
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  misr_enable_r <= 1'b0;
	else
	  if (misr_enable_set == 1'b1)
	    misr_enable_r <= 1'b1;
	  else if (testcounter_rst == 1'b1)
	    misr_enable_r <= 1'b0;
     end
   
   
   /*
    * pattern counter reg
    * 
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  pattern_counter_r <= {{PATTERNCOUNTER_BITS}{1'b0}};
	else
	  if (patterncounter_inc == 1'b1)
	    pattern_counter_r <= pattern_counter_r + 1;
	  else if (patterncounter_rst == 1'b1)
	    pattern_counter_r <= {{PATTERNCOUNTER_BITS}{1'b0}};
     end // always @ (posedge bist_clk)

   /*
    * bit counter reg
    *
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  bit_counter_r <= {{SHIFTCOUNTER_BITS+1}{1'b0}};
	else
	  if (bitcounter_inc == 1'b1)
	    bit_counter_r <= bit_counter_r + 1;
	  else if (bitcounter_rst == 1'b1)
	    bit_counter_r <={{SHIFTCOUNTER_BITS+1}{1'b0}};
     end

   /*
    * test counter 
    *
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  test_counter_r <= {{CONFCOUNTERSIZE}{1'b0}};
	else
	  if (testcounter_inc == 1'b1)
	    test_counter_r <= test_counter_r + 1;
	  else if (testcounter_rst == 1'b1)
	    test_counter_r <= {{CONFCOUNTERSIZE}{1'b0}};
     end

   /*
    * weighting output register
    * 
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  bist_weighting_sel <= {{WEIGHTING_SEL}{1'b0}};
	else
	  if(bist_weight_load == 1'b1)
	    bist_weighting_sel <= bist_weight_sel;
     end

   /*
    * patterns to apply (config register)
    *
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  pattern_to_apply_r <= PATTERNS;
	else
	  if (bist_scan_conf_en == 1'b1)
	    pattern_to_apply_r <= {pattern_to_apply_r[PATTERNCOUNTER_BITS-2:0],bist_scan_conf_in};
     end
   
   /*
    * bits to apply (config register)
    *
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  bits_to_apply_r <= SHIFT_BITS;
	else
	  if (bist_scan_conf_en == 1'b1)
	    bits_to_apply_r <= {bits_to_apply_r[SHIFTCOUNTER_BITS-2:0],pattern_to_apply_r[PATTERNCOUNTER_BITS-1]};
     end

   /*
    * conf (config register)
    *
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  conf_r <= BIST_CONF;
	else
	  if (bist_scan_conf_en == 1'b1)
	    conf_r <= {conf_r[CONFCOUNTER-2:0],bits_to_apply_r[SHIFTCOUNTER_BITS-1]};
     end

   assign bist_delaytest = conf_r[0];
   
   /*
    * flipping conf r (flipping configuration reg)
    *
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if (bist_rst_n == 1'b0)
	  flipping_conf_r <= {{NUM_SCAN_CHAINS}{1'b1}};
	else
	  if (bist_scan_conf_en == 1'b1)
	    flipping_conf_r <= {flipping_conf_r[NUM_SCAN_CHAINS-2:0],conf_r[CONFCOUNTER-1]};
     end

   
   /*
    * flipping output register
    * 
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if(bist_rst_n == 1'b0)
	  bist_flipping_sel <= {{NUM_SCAN_CHAINS}{1'b0}};
	else
	  if(bist_flip_load == 1'b1)
	    if (bist_flip_en == 1'b1)
	      bist_flipping_sel <= flipping_conf_r;
	    else
	      bist_flipping_sel <= {{NUM_SCAN_CHAINS}{1'b0}};
     end // always @ (bist_clk)
   

   
   bistcontroller_fsm U_BISTCONTROLLER_FSM(/*AUTOINST*/
                                           // Outputs
                                           .bist_done           (bist_done),
                                           .bist_en             (bist_en),
                                           .bist_scan_en        (bist_scan_en),
                                           .bist_lfsr_en        (bist_lfsr_en),
                                           .bist_misr_en        (bist_misr_en),
                                           .bist_weight_sel     (bist_weight_sel[WEIGHTING_SEL-1:0]),
                                           .bist_weight_load    (bist_weight_load),
                                           .bist_flip_en        (bist_flip_en),
                                           .bist_flip_load      (bist_flip_load),
                                           .bitcounter_inc      (bitcounter_inc),
                                           .patterncounter_inc  (patterncounter_inc),
                                           .testcounter_inc     (testcounter_inc),
                                           .patterncounter_rst  (patterncounter_rst),
                                           .bitcounter_rst      (bitcounter_rst),
                                           .testcounter_rst     (testcounter_rst),
                                           .misr_enable_set     (misr_enable_set),
                                           // Inputs
                                           .bist_clk            (bist_clk),
                                           .bist_rst_n          (bist_rst_n),
                                           .bist_start          (bist_start),
                                           .bit_counter_r       (bit_counter_r[SHIFTCOUNTER_BITS-1:0]),
                                           .bits_to_apply_r     (bits_to_apply_r[SHIFTCOUNTER_BITS-1:0]),
                                           .pattern_counter_r   (pattern_counter_r[PATTERNCOUNTER_BITS-1:0]),
                                           .pattern_to_apply_r  (pattern_to_apply_r[PATTERNCOUNTER_BITS-1:0]),
                                           .test_counter_r      (test_counter_r[CONFCOUNTERSIZE-1:0]),
                                           .conf_r              (conf_r[CONFCOUNTER-1:0]),
                                           .misr_enable_r       (misr_enable_r));
   
	
endmodule // bistcontroller

/*
 Local Variables:
 verilog-library-directories:(
 ".")
 End:
*/

