//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  bistcontroller_fsm.v
//  Revision        :  $Revision: 1.0 $
//  Checkin Date    :  $Date: 2008/05/28 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <bistcontroller_fsm>
//  Description     :  <finite state machine of the bist controller>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module bistcontroller_fsm(/*AUTOARG*/
  // Outputs
  bist_done, bist_en, bist_scan_en, bist_lfsr_en, bist_misr_en,
  bist_weight_sel, bist_weight_load, bist_flip_en, bist_flip_load,
  bitcounter_inc, patterncounter_inc, testcounter_inc, patterncounter_rst,
  bitcounter_rst, testcounter_rst, misr_enable_set,
  // Inputs
  bist_clk, bist_rst_n, bist_start, bit_counter_r, bits_to_apply_r,
  pattern_counter_r, pattern_to_apply_r, test_counter_r, conf_r,
  misr_enable_r
  );

   `include "chip_params.v"
   `include "bistcontroller_params.v"
   
   //outputs
   output                       bist_done;
   output                       bist_en;
   output                       bist_scan_en;
   output                       bist_lfsr_en;
   output                       bist_misr_en;
   output [WEIGHTING_SEL-1:0]   bist_weight_sel;
   output 			bist_weight_load;
   output 			bist_flip_en;
   output                       bist_flip_load;
   output 			bitcounter_inc;
   output 			patterncounter_inc;
   output 			testcounter_inc;
   output 			patterncounter_rst;
   output 			bitcounter_rst;
   output 			testcounter_rst;
   output 			misr_enable_set;
			
   
   //input
   input 			 bist_clk;
   input 			 bist_rst_n;   
   input 			 bist_start;
   input [SHIFTCOUNTER_BITS-1:0] bit_counter_r;
   input [SHIFTCOUNTER_BITS-1:0] bits_to_apply_r;
   input [PATTERNCOUNTER_BITS-1:0] pattern_counter_r;
   input [PATTERNCOUNTER_BITS-1:0] pattern_to_apply_r;
   input [CONFCOUNTERSIZE-1:0] 	 test_counter_r;
   input [CONFCOUNTER-1:0] 	 conf_r;
   input 			 misr_enable_r;
   
   /*AUTOWIRE*/

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   reg                  bist_done;
   reg                  bist_en;
   reg                  bist_flip_en;
   reg                  bist_flip_load;
   reg                  bist_lfsr_en;
   reg                  bist_misr_en;
   reg                  bist_scan_en;
   reg                  bist_weight_load;
   reg [WEIGHTING_SEL-1:0] bist_weight_sel;
   reg                  bitcounter_inc;
   reg                  bitcounter_rst;
   reg                  misr_enable_set;
   reg                  patterncounter_inc;
   reg                  patterncounter_rst;
   reg                  testcounter_inc;
   reg                  testcounter_rst;
   // End of automatics

   
   //FSM registers
   parameter [3:0] 		 IDLE            = 0,
				 LOAD_CONF       = 1,
				 SCAN_IN_OUT     = 2,
				 WAIT0           = 3, //if bist controller runs
				 WAIT1           = 4, //at-speed, 4 waits are needed
				 WAIT2           = 5, //if it runs at at-speed/2 only 2 are needed
				 WAIT3           = 6; //this thas to duw with the clock mux and controller
   
   
   reg [3:0] 			 currentstate;
   reg [3:0] 			 nextstate;

   /*
    * Current state register
    * 
    */
   always @ (posedge bist_clk or negedge bist_rst_n)
     begin
	if(bist_rst_n == 'b0)
	  currentstate <= IDLE;
	else
	  currentstate <= nextstate;
     end
   
   /*
    * Final state machine
    *
    */
   always @ (/*AUTOSENSE*/WEIGHTING_SEL or bist_start or bit_counter_r
             or bits_to_apply_r or conf_r or currentstate or misr_enable_r
             or pattern_counter_r or pattern_to_apply_r or test_counter_r)
     begin
	//some default output values
	bist_done          = 1'b0;
        bist_en            = 1'b0;
        bist_scan_en       = 1'b0;
        bist_lfsr_en       = 1'b0;
        bist_misr_en       = 1'b0;
	bist_weight_sel    = {{WEIGHTING_SEL}{1'b0}};
   	bist_weight_load   = 1'b0;
	bist_flip_en       = 1'b0; 
        bist_flip_load     = 1'b0;
	bitcounter_inc     = 1'b0;
	patterncounter_inc = 1'b0;
	testcounter_inc    = 1'b0;
	patterncounter_rst = 1'b0;
	bitcounter_rst     = 1'b0;
	testcounter_rst    = 1'b0;
	misr_enable_set    = 1'b0;
	
	
	nextstate = currentstate;
	
	case (currentstate)
	  IDLE:
	    begin
	       if (bist_start == 1'b1)
		 begin
		    nextstate = LOAD_CONF;
		    //reset test counter
		    testcounter_rst = 1'b1;
		 end
	    end
	  LOAD_CONF:
	    begin
	       bist_en             = 1'b1;
	       testcounter_inc     = 1'b1;
	       //reset pattern counter;
	       patterncounter_rst = 1'b1;
	       
	       if(test_counter_r == 0)
		 begin
		    //test if plain test is to be run
		    if(conf_r[1])
		      begin
			 nextstate = SCAN_IN_OUT;
			 //set other conf bits
		      end
		 end
	       else if(test_counter_r == 1)
		 begin
		    //weight 1/4
		    if(conf_r[2])
		      begin
			 nextstate = SCAN_IN_OUT;
			 //set other conf bits
			 bist_weight_sel      = 1;
			 bist_weight_load = 1'b1;
			 bist_flip_en         = 1'b0;
			 bist_flip_load       = 1'b1;
		      end
		 end
	       else if(test_counter_r == 2)
		 begin
		    //weight 3/4
		    if(conf_r[3])
		      begin
			 nextstate = SCAN_IN_OUT;
			 //set other conf bits
			 bist_weight_sel      = 1;
			 bist_weight_load = 1'b1;
			 bist_flip_en         = 1'b1;
			 bist_flip_load       = 1'b1;
		      end
		 end
	       else if(test_counter_r == 3)
		 begin
		    //weight 1/8
		    if(conf_r[4])
		      begin
			 nextstate = SCAN_IN_OUT;
			 //set other conf bits
			 bist_weight_sel      = 2;
			 bist_weight_load = 1'b1;
			 bist_flip_en         = 1'b0;
			 bist_flip_load       = 1'b1;
		      end
		 end
	       else if(test_counter_r == 4)
		 begin
		    //weight 7/8
		    if(conf_r[5])
		      begin
			 nextstate = SCAN_IN_OUT;
			 //set other conf bits
			 bist_weight_sel      = 2;
			 bist_weight_load = 1'b1;
			 bist_flip_en         = 1'b1;
			 bist_flip_load       = 1'b1;
		      end
		 end
	       else if(test_counter_r == 5)
		 begin
		    //weight 1/16
		    if(conf_r[6])
		      begin
			 nextstate = SCAN_IN_OUT;
			 //set other conf bits
			 bist_weight_sel      = 3;
			 bist_weight_load = 1'b1;
			 bist_flip_en         = 1'b0;
			 bist_flip_load       = 1'b1;
		      end
		 end
	       else if(test_counter_r == 6)
		 begin
		    //weight 15/16
		    if(conf_r[7])
		      begin
			 nextstate = SCAN_IN_OUT;
			 //set other conf bits
			 bist_weight_sel      = 3;
			 bist_weight_load = 1'b1;
			 bist_flip_en         = 1'b1;
			 bist_flip_load       = 1'b1;
		      end
		 end
	       else
		 begin
		    nextstate = IDLE;
		    bist_done = 1'b1;
		 end
	    end
	  SCAN_IN_OUT:
	    begin
	       bist_en = 1'b1;
	       
	       if(bits_to_apply_r == bit_counter_r)
		 begin
		    misr_enable_set = 1'b1;
		    if(conf_r[0])
		      nextstate = WAIT0;
		    else
		      nextstate = WAIT1;
		 end
	       else
		 begin
		    bist_scan_en = 1'b1;
		    bitcounter_inc = 1'b1;
		    bist_lfsr_en = 1'b1;
		    if(misr_enable_r == 1'b1)
		      bist_misr_en = 1'b1;
		end 
	    end 
	  WAIT0:
	    begin
	       bist_en = 1'b1;
	       nextstate = WAIT1;
	    end
	  WAIT1:
	    begin
	       bist_en = 1'b1;
	       nextstate = WAIT2;
	    end
	  WAIT2:
	    begin
	       bist_en = 1'b1;
	       nextstate = WAIT3;
	    end
	  WAIT3:
	    begin
	       bist_en = 1'b1;
	       //inc pattern counter
	       patterncounter_inc = 1'b1;
	       //reset bit counter
	       bitcounter_rst = 1'b1;
	       //nextstate
	       if (pattern_counter_r == pattern_to_apply_r)
		 nextstate = LOAD_CONF;
	       else
		 nextstate = SCAN_IN_OUT;
	    end
	  default:
	    begin
	       nextstate = IDLE;
	    end
	endcase // case(currentstate)
     end // always @ (...

endmodule // bistcontroller_fsm
