//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  chip.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/26 17:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <chip>
//  Description     :  <design top module>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <03.2008>
//
//
//****************************************************************************

module chip(/*AUTOARG*/
  // Outputs
  bist_done, conf_scan_out, scan_inputs, scan_en, ext_scan_out, sys_clk,
  // Inputs
  rst_n, ext_clk, bist_start, ext_test_en, ext_scan_en, ext_scan_in,
  conf_scan_en, conf_scan_in, scan_outputs, osc_clk
  );

`include "chip_params.v"
  
   //outputs
   output                           bist_done;
   output                           conf_scan_out; //from bistc. and clockc.
   output [NUM_SCAN_CHAINS-1:0]     scan_inputs;
   output 			    scan_en;
   output [NUM_EXT_SCAN_CHAINS-1:0] ext_scan_out;
   output                           sys_clk;
  
   //inputs
   input 			    rst_n;         //general inputs
   input 			    ext_clk;
   input 			    bist_start;    //start bist
   input 			    ext_test_en;
   input 			    ext_scan_en;
   input [NUM_EXT_SCAN_CHAINS-1:0]  ext_scan_in;   //external scan chains_in			    
   input 			    conf_scan_en;  //config signals for bistc. and clockc.
   input 			    conf_scan_in;
   input [NUM_SCAN_CHAINS-1:0]      scan_outputs;

   input                            osc_clk;
  
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire                 bist_clk;               // From U_CLOCKCONTROLLER of clockcontroller.v
   wire                 bist_en;                // From U_BISTTOP of bisttop.v
   wire                 bist_scan_en;           // From U_BISTTOP of bisttop.v
   wire [NUM_SCAN_CHAINS-1:0] bist_scan_inputs; // From U_BISTTOP of bisttop.v
   wire                 bisttop_scan_conf_out;  // From U_BISTTOP of bisttop.v
   wire                 clockconf_scan_out;     // From U_CLOCKCONTROLLER of clockcontroller.v
   // End of automatics
  
   /*AUTOREG*/

   wire 			clk;
   wire 			scan_en;
   
   wire [NUM_SCAN_CHAINS-1:0] 	bist_scan_outputs;
   wire  [NUM_SCAN_CHAINS-1:0] 	scan_inputs;
   wire                         bist_rst_n;
   wire                         bisttop_scan_conf_en;
   wire                         clockconf_scan_en;
   wire                         clockconf_scan_in;
   wire                         bisttop_scan_conf_in;
  
   assign 			bist_rst_n = rst_n;
   assign 			clk = sys_clk;
   assign 			scan_en = bist_scan_en | ext_scan_en;
   
  assign 			bist_scan_outputs = scan_outputs;
  assign 			ext_scan_out = scan_outputs[NUM_SCAN_CHAINS-1:NUM_SCAN_CHAINS-NUM_EXT_SCAN_CHAINS];
          
   //Connecting config scan signals between clockc. and bistc. and in/out-puts.
   assign 			bisttop_scan_conf_en = conf_scan_en;
   assign 			clockconf_scan_en = conf_scan_en;
   assign 			clockconf_scan_in = conf_scan_in;
   assign 			bisttop_scan_conf_in = clockconf_scan_out;
   assign 			conf_scan_out = bisttop_scan_conf_out;

   //Connecting N internal scan chains to M external scan chains, N > M, with muxes
   mux #(NUM_EXT_SCAN_CHAINS) U_MUX_0 (
                                       .out (scan_inputs[NUM_EXT_SCAN_CHAINS-1:0]),
                                       .sel (bist_en),
                                       .ina (bist_scan_inputs[NUM_EXT_SCAN_CHAINS-1:0]),
                                       .inb (ext_scan_in));

  generate
    genvar                      i;
    for ( i = 1; i < SCAN_CHAIN_MAP; i = i +1)
      begin : muxing
        mux #(NUM_EXT_SCAN_CHAINS) U_MUX_ (
                                           .out (scan_inputs[NUM_EXT_SCAN_CHAINS*(i+1)-1:NUM_EXT_SCAN_CHAINS*i]),
                                           .sel (bist_en),
                                           .ina (bist_scan_inputs[NUM_EXT_SCAN_CHAINS*(i+1)-1:NUM_EXT_SCAN_CHAINS*i]),
                                           .inb (scan_outputs[NUM_EXT_SCAN_CHAINS*i-1:NUM_EXT_SCAN_CHAINS*(i-1)]));
      end
    endgenerate
       
   //BIST CONTROLLER, PATTERNGENERATOR AND ANALYSATOR
   bisttop U_BISTTOP(/*AUTOINST*/
                     // Outputs
                     .bist_en           (bist_en),
                     .bist_done         (bist_done),
                     .bist_scan_en      (bist_scan_en),
                     .bist_scan_inputs  (bist_scan_inputs[NUM_SCAN_CHAINS-1:0]),
                     .bisttop_scan_conf_out(bisttop_scan_conf_out),
                     // Inputs
                     .bist_clk          (bist_clk),
                     .bist_rst_n        (bist_rst_n),
                     .bist_start        (bist_start),
                     .bist_scan_outputs (bist_scan_outputs[NUM_SCAN_CHAINS-1:0]),
                     .bisttop_scan_conf_in(bisttop_scan_conf_in),
                     .bisttop_scan_conf_en(bisttop_scan_conf_en));

   //CLOCK CONTROLLER
   clockcontroller U_CLOCKCONTROLLER(/*AUTOINST*/
                                     // Outputs
                                     .sys_clk           (sys_clk),
                                     .bist_clk          (bist_clk),
                                     .clockconf_scan_out(clockconf_scan_out),
                                     // Inputs
                                     .ext_clk           (ext_clk),
                                     .osc_clk           (osc_clk),
                                     .rst_n             (rst_n),
                                     .bist_en           (bist_en),
                                     .ext_test_en       (ext_test_en),
                                     .bist_scan_en      (bist_scan_en),
                                     .ext_scan_en       (ext_scan_en),
                                     .clockconf_scan_in (clockconf_scan_in),
                                     .clockconf_scan_en (clockconf_scan_en));

endmodule // chip

/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../include"
 "../../../clockcontroller/custom/verilog/"
 "../../../bisttop/custom/verilog/"
 "../../../../../verification/chip_tb/")
 End:
*/
//"../../../dut/custom/verilog/")
//"../../../../../verification/chip_tb/"
