//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  clockconroller.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/26 17:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <clockcontroller>
//  Description     :  <selects and make clock signals>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <03.2008>
//
//
//****************************************************************************

module clockcontroller(/*AUTOARG*/
   // Outputs
   sys_clk, bist_clk, clockconf_scan_out, 
   // Inputs
   ext_clk, osc_clk, rst_n, bist_en, ext_test_en, bist_scan_en, 
   ext_scan_en, clockconf_scan_in, clockconf_scan_en
   );

   `include "clockcontroller_params.v"
   
   //outputs
   output sys_clk;
   output bist_clk;
   output clockconf_scan_out;
   
   
   //inputs
   input ext_clk;           //clock signals
   input osc_clk; 
   input rst_n;             //reset signal
   input bist_en;           //test enable signals
   input ext_test_en;
   input bist_scan_en;      //scan enable signals
   input ext_scan_en;
   input clockconf_scan_in; //clock config signals
   input clockconf_scan_en;
   
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   // End of automatics

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   // End of automatics
   
   //
   reg [6:0] osc_clockdivider;// = 7'h00; // nesessary for simulation
   reg [SCAN_EN_PIPELINE_SIZE-1:0] scan_enable_pipeline;
   reg [7:0] 			   clockconf;

   wire [1:0] 			   mode;
   wire [2:0] 			   osc_clk_div_sel;
   wire [2:0] 			   bist_scan_clk_div_sel;

   wire 			   bist_clk;
   wire 			   sys_clk;
   wire 			   lc_clk;   
   wire 			   test_clk;
   wire 			   ext_scan_clk;
   wire 			   scan_clk;

   wire 			   scan_en;
   reg 				   osc_clk_div;
   reg 				   bist_scan_clk;
   
   wire 			   test_en;
   wire 			   scan_enable_pipeline_sel;
     

   initial osc_clockdivider = 7'h00;
   
   
   assign 		ext_scan_clk = ext_clk;
   
   assign 		mode = clockconf[1:0];
   assign 		osc_clk_div_sel = clockconf[4:2];
   assign 		bist_scan_clk_div_sel = clockconf[7:5];
   assign 		clockconf_scan_out = clockconf[7];
  

   
   /*
    * Clock configuration register
    * 
    */
   always @ (posedge bist_clk or negedge rst_n) // or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  clockconf <= DEFAULT_CLOCK_CONFIG; //default to slow clocks?
	else
	  if (clockconf_scan_en == 1'b1)
	    clockconf <= {clockconf[6:0],clockconf_scan_in};
	  //else add some modification to mode regs from bist controller
     end

   /*
    * Clock divider, implemented as a counter,
    * THIS COUNTER CAN NOT HAVE A RESET, AS THIS WILL STOP THE CLOCK
    * or can it as the reset now is async?
    */
   always @ (posedge osc_clk)
     begin
	osc_clockdivider <= osc_clockdivider + 1;
     end

   /*
    * MUX for selecting the main system clock
    * 
    */
   always @ (/*AUTOSENSE*/osc_clk or osc_clk_div_sel
	     or osc_clockdivider)
     begin
	case (osc_clk_div_sel)
	  0: osc_clk_div = osc_clk;
	  1: osc_clk_div = osc_clockdivider[0];
	  2: osc_clk_div = osc_clockdivider[1];
	  3: osc_clk_div = osc_clockdivider[2];
	  4: osc_clk_div = osc_clockdivider[3];
	  5: osc_clk_div = osc_clockdivider[4];
	  6: osc_clk_div = osc_clockdivider[5];
	  7: osc_clk_div = osc_clockdivider[6];
	  default: osc_clk_div = osc_clk;
	endcase // case(osc_clk_div_sel)
     end

   /*
    * MUX for selecting the scan clock for BIST,
    * must be equal or slower than osc_clk_div
    */
   always @ (/*AUTOSENSE*/bist_scan_clk_div_sel or osc_clk
	     or osc_clockdivider)
     begin
	case (bist_scan_clk_div_sel)
	  0: bist_scan_clk = osc_clk;
	  1: bist_scan_clk = osc_clockdivider[0];
	  2: bist_scan_clk = osc_clockdivider[1];
	  3: bist_scan_clk = osc_clockdivider[2];
	  4: bist_scan_clk = osc_clockdivider[3];
	  5: bist_scan_clk = osc_clockdivider[4];
	  6: bist_scan_clk = osc_clockdivider[5];
	  7: bist_scan_clk = osc_clockdivider[6];
	  default: bist_scan_clk = osc_clk;
	endcase // case(osc_clk_div_sel)
     end

   /*
    * The SCAN ENABLE PIPELINE
    * used to add wait cycles to make sure the scan enable
    * signal is low before launch and capture. Also used to 
    * wait for the last clock periode of the scan to finish.
    */
   always @ (posedge osc_clk_div or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  scan_enable_pipeline <= {{SCAN_EN_PIPELINE_SIZE}{1'b0}};
	else
	  scan_enable_pipeline <= {scan_enable_pipeline[SCAN_EN_PIPELINE_SIZE-2:0],scan_en};
     end

   /*
    * MUX controll signals. 
    * scan_enable_pipeline_sel selects one/two at-speed clocks during launch/capture
    * test_en selects between system clock and test clock
    * scan_en selects between scan clock and the clock to use during launch/capture
    */
   assign scan_enable_pipeline_sel = ((scan_enable_pipeline[SCAN_EN_PIPELINE_SIZE-2] &
		 ~scan_enable_pipeline[SCAN_EN_PIPELINE_SIZE-3] & mode[0]) | 
		(scan_enable_pipeline[SCAN_EN_PIPELINE_SIZE-1] & 
		 ~scan_enable_pipeline[SCAN_EN_PIPELINE_SIZE-2] & mode[1]));

   assign test_en = bist_en | ext_test_en;
   assign scan_en = bist_scan_en | ext_scan_en;

   
   clockmux2 U_CLOCKMUX2_0(
			//outputs
			.clk_out (sys_clk),
			//inputs
			.clk1    (test_clk),
			.clk2    (osc_clk_div),
			.clk_sel (test_en));

   clockmux2 U_CLOCKMUX2_1(
			//outputs
			.clk_out (lc_clk),
			//inputs
			.clk1    (osc_clk_div),
			.clk2    (1'b0),
			.clk_sel (scan_enable_pipeline_sel));
   
   clockmux2 U_CLOCKMUX2_2(
			//outputs
			.clk_out (test_clk),
			//inputs
			.clk1    (scan_clk),
			.clk2    (lc_clk),
			.clk_sel (scan_en));

   /*
    * Cange the bist clock during external scan so that the bist controller
    * can be scan tested and new config can be scanned in/out of 
    * the bist controller
    * 
    * assign bist_clk = bist_scan_clk;
    */
   clockmux2 U_CLOCKMUX2_3(
			//outputs
			.clk_out (bist_clk),
			//inputs
			.clk1    (ext_clk),
			.clk2    (bist_scan_clk),
			.clk_sel (ext_test_en));

   /*
    *
    * assign scan_clk = bist_en ? bist_scan_clk : ext_scan_clk;
    */
   clockmux2 U_CLOCKMUX2_4(
			//outputs
			.clk_out (scan_clk),
			//inputs
			.clk1    (bist_scan_clk),
			.clk2    (ext_scan_clk),
			.clk_sel (bist_en));
   
   

   
endmodule // clockcontroller



/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../include"
 "../../rtl/module/trunk/custom/verilog/")
 End:
*/