//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  alu.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <alu>
//  Description     :  <the alu>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module alu(/*AUTOARG*/
   // Outputs
   alu_regfile_data, alu_spi_data, alu_statusreg, 
   alu_regfile_write_en, alu_spi_write_en, 
   // Inputs
   regfile_alu_a, regfile_alu_b, regfile_alu_d, spi_alu_data, 
   alu_opcode, statusreg_alu
   );

   `include "spi_co_processor_params.v"

   //outputs
   output [CPUBITS-1:0]         alu_regfile_data;
   output [CPUBITS-1:0] 	alu_spi_data; 	
   output [STATUSREGBITS-1:0] 	alu_statusreg;
   output 			alu_regfile_write_en;
   output 			alu_spi_write_en;
   
   //inputs
   input [CPUBITS-1:0]   	regfile_alu_a;
   input [CPUBITS-1:0]   	regfile_alu_b;
   input [CPUBITS-1:0] 		regfile_alu_d; 		
   input [CPUBITS-1:0] 		spi_alu_data;
   input [OPCODEBITS-1:0] 	alu_opcode;
   input [STATUSREGBITS-1:0] 	statusreg_alu;
 			
		      
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   // End of automatics

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   reg [CPUBITS-1:0]	alu_regfile_data;
   reg			alu_regfile_write_en;
   reg			alu_spi_write_en;
   reg [STATUSREGBITS-1:0]alu_statusreg;
   // End of automatics

   assign 		alu_spi_data = alu_regfile_data;
   
   always @ (/*AUTOSENSE*/ADC or ADD or AND or CPUBITS or INV or LSL
	     or LSR or MAC or MOV or MUL or NOP or OR or READ
	     or READSREG or ROL or ROR or SBC or SREG_C or SUB
	     or WRITE or WRITESREG or XOR or alu_opcode
	     or regfile_alu_a or regfile_alu_b or regfile_alu_d
	     or spi_alu_data or statusreg_alu)
     begin
	//default values
	alu_regfile_write_en = 1'b0;
	alu_spi_write_en = 1'b0;
	alu_regfile_data = {{CPUBITS}{1'b0}};
	alu_statusreg = statusreg_alu; //output the last statusreg for NOP
	
	case (alu_opcode)
	  NOP:
	    begin
	       //do nothing
	    end
	  WRITE:
	    begin
	       //
	       alu_regfile_write_en = 1'b1;
	       alu_regfile_data = spi_alu_data;
	    end
	  READ:
	    begin
	       //
	       alu_regfile_data = regfile_alu_a;
	       alu_spi_write_en  = 1'b1;
	    end
	  WRITESREG:
	    begin
	       //
	       alu_statusreg = spi_alu_data[STATUSREGBITS-1:0];
	    end
	  READSREG:
	    begin
	       //
	       alu_regfile_data[STATUSREGBITS-1:0] = statusreg_alu;
	       alu_spi_write_en = 1'b1;
	    end
	  MOV:
	    begin
	       //
	       alu_regfile_data = regfile_alu_a;
	       alu_regfile_write_en = 1'b1;
	    end
	  ADD:
	    begin
	       //Z,C,N,V,S,H
	       {alu_statusreg[SREG_C],alu_regfile_data} = regfile_alu_a + regfile_alu_b;
	       alu_regfile_write_en = 1'b1; 
	    end
	  ADC:
	    begin
	       //Z,C,N,V,S,H
	       {alu_statusreg[SREG_C],alu_regfile_data} = regfile_alu_a + regfile_alu_b + statusreg_alu[SREG_C];
	       alu_regfile_write_en = 1'b1;
	    end
	  SUB:
	    begin
	       //Z,C,N,V,S,H
	       {alu_statusreg[SREG_C],alu_regfile_data} = regfile_alu_a - regfile_alu_b;
	       alu_regfile_write_en = 1'b1;
	    end
	  SBC:
	    begin
	       //Z,C,N,C,S,H
	       {alu_statusreg[SREG_C],alu_regfile_data} = regfile_alu_a - regfile_alu_b - statusreg_alu[SREG_C];
	       alu_regfile_write_en = 1'b1;
	    end
	  MUL:
	    begin
	       //Z,C
	       {alu_statusreg[SREG_C],alu_regfile_data} = regfile_alu_a * regfile_alu_b;
	       alu_regfile_write_en = 1'b1;
	    end
	  //DIV:
	  //  begin
	  //     {alu_statusreg[SREG_C],alu_regfile_data} = regfile_alu_a / regfile_alu_b;
	  //  end
	  MAC:
	    begin
	       //Z,C,N,S,H
	       {alu_statusreg[SREG_C],alu_regfile_data} = (regfile_alu_a * regfile_alu_b) + regfile_alu_d;
	       alu_regfile_write_en = 1'b1;
	    end
	  LSL:
	    begin
	       //Z,C,N,V,H
	       {alu_statusreg[SREG_C],alu_regfile_data} = {regfile_alu_a,1'b0};
	       alu_regfile_write_en = 1'b1;
	    end
	  LSR:
	    begin
	       //Z,C,N,V
	       alu_regfile_data = {1'b0,regfile_alu_a[CPUBITS-1:1]};
	       alu_statusreg[SREG_C] = regfile_alu_a[0];
	       alu_regfile_write_en = 1'b1;
	    end
	  ROL:
	    begin
	       //Z,C,N,V,H
	       {alu_statusreg[SREG_C],alu_regfile_data} = {regfile_alu_a,statusreg_alu[SREG_C]};
	       alu_regfile_write_en = 1'b1;
	    end
	  ROR:
	    begin
	       //Z,C,N,V
	       alu_regfile_data = {statusreg_alu[SREG_C],regfile_alu_a[CPUBITS-1:1]};
	       alu_statusreg[SREG_C] = regfile_alu_a[0];
	       alu_regfile_write_en = 1'b1;
	    end
	  AND:
	    begin
	       //Z,N,V,S
	       alu_regfile_data = regfile_alu_a & regfile_alu_b;
	       alu_regfile_write_en = 1'b1;
	    end
	  OR:
	    begin
	       //Z,N,V,S
	       alu_regfile_data = regfile_alu_a | regfile_alu_b;
	       alu_regfile_write_en = 1'b1;
	    end
	  XOR:
	    begin
	       //Z,N,V,S
	       alu_regfile_data = regfile_alu_a ^ regfile_alu_b;
	       alu_regfile_write_en = 1'b1;
	    end
	  INV:
	    begin
	       //Z,C,N,V,S,H
	       alu_regfile_data = ~regfile_alu_a;
	       alu_regfile_write_en = 1'b1;
	    end
	  //add instructions to set/clear bits in SREG if extended to support more SREG bits.
	  default
	    begin
	    end
	endcase // case(alu_opcode)
     end // always @ (...
   
   
endmodule // alu
