//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  spi_co_proc_fsm.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <spi_co_proc_fsm>
//  Description     :  <finite state machine controlling spi and alu>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module spi_co_proc_fsm(/*AUTOARG*/
   // Outputs
   fsm_spi_load, fsm_spi_load_inc, fsm_instrbuf_msb_load, 
   fsm_instrbuf_lsb_load, fsm_databuf_load, fsm_instrreg_load, 
   fsm_instrreg_load_fast, debug, 
   // Inputs
   clk, rst_n, spi_fsm_load, ss_n_sync, instrbuf, spi_data, 
   fsm_data_finish
   );

   `include "spi_co_processor_params.v"
   
   //outputs
   output       fsm_spi_load;
   output 	fsm_spi_load_inc;
   output       fsm_instrbuf_msb_load;
   output       fsm_instrbuf_lsb_load;
   output 	fsm_databuf_load;
   output 	fsm_instrreg_load;
   output 	fsm_instrreg_load_fast;
   output [2:0] debug;
   
   
   //inputs
   input 	         clk;
   input 	         rst_n;
   input 	         spi_fsm_load;
   input 	         ss_n_sync;
   input [INSTRBITS-1:0] instrbuf;
   input [SPIBITS-1:0] 	 spi_data;
   input 		 fsm_data_finish;

   
   
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   // End of automatics

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   reg			fsm_databuf_load;
   reg			fsm_instrbuf_lsb_load;
   reg			fsm_instrbuf_msb_load;
   reg			fsm_instrreg_load;
   reg			fsm_instrreg_load_fast;
   reg			fsm_spi_load;
   reg			fsm_spi_load_inc;
   // End of automatics

   //other regs and wires
   
   //FSM registers
   parameter [2:0] 		 IDLE            = 0,
				 INSTRMSB        = 1,
				 RINSTRLSB       = 2,
				 WINSTRLSB       = 3,
				 INSTRLSB        = 4,
				 RDATA           = 5,
				 RXDATA          = 6,
				 WDATA           = 7;
   
   reg [2:0] 			 currentstate;
   reg [2:0] 			 nextstate;

   assign 			 debug = currentstate;
   
   /*
    * Current state register
    * 
    */
   always @ (posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  currentstate <= IDLE;
	else
	  currentstate <= nextstate;
     end
   
   /*
    * FSM combinational part
    * 
    */
   always @ (/*AUTOSENSE*/READ or SPIBITS or WRITE or currentstate
	     or fsm_data_finish or spi_data or spi_fsm_load
	     or ss_n_sync)
     begin
	//default outputs
	fsm_instrbuf_msb_load = 1'b0;
	fsm_instrbuf_lsb_load = 1'b0;
	fsm_databuf_load      = 1'b0;
	fsm_spi_load          = 1'b0;
	fsm_spi_load_inc      = 1'b0;
	fsm_instrreg_load     = 1'b0;
	fsm_instrreg_load_fast= 1'b0;
	nextstate             = currentstate;
	
	case (currentstate)
	  IDLE:
	    begin
	       //wait for ss_n_sync to go low and start RX/TX
	       //reset/set counter is done by the ss_n signal
	       if (ss_n_sync == 1'b0)
		 nextstate = INSTRMSB;
	       else
		 nextstate = IDLE;
	    end
	  INSTRMSB:
	    begin
	       //the first package is the MSBs of the instr
	       //wait to they are received and store them
	       //also check what the MSBs are and decide
	       //what do do
	       if (ss_n_sync == 1'b0)
		 begin
		    if (spi_fsm_load == 1'b1)
		      begin
			 fsm_instrbuf_msb_load = 1'b1;
			 if (spi_data[SPIBITS-1:1] == READ | spi_data[SPIBITS-1:1] == READSREG)
			   nextstate = RINSTRLSB;
			 else
			   begin
			      if (spi_data[SPIBITS-1:1] == WRITE)
				nextstate = WINSTRLSB;
			      else
				nextstate = INSTRLSB;
			   end
		      end // if (spi_fsm_load == 1'b1)
		 end // if (ss_n_sync == 1'b0)
	       else
		 nextstate = IDLE;
	    end
	  RINSTRLSB:
	    begin
	       //receive the LSBs of the instr
	       //and store it and send it to CPU
	       if (ss_n_sync == 1'b0)
		 begin
		    if (spi_fsm_load == 1'b1)
		      begin
			 fsm_instrbuf_lsb_load = 1'b1;
			 fsm_instrreg_load_fast = 1'b1;
			 nextstate = RDATA;
		      end
		 end
	       else
		 nextstate = IDLE;
	    end
	  WINSTRLSB:
	    begin
	       if (ss_n_sync == 1'b0)
		 begin
		    if (spi_fsm_load == 1'b1)
		      begin
			 fsm_instrbuf_lsb_load = 1'b1;
			 nextstate = WDATA;
		      end
		 end
	       else
		 nextstate = IDLE;
	    end
	  INSTRLSB:
	    begin
	       if (ss_n_sync == 1'b0)
		 begin
		    if (spi_fsm_load == 1'b1)
		      begin
			 fsm_instrbuf_lsb_load = 1'b1;
			 fsm_instrreg_load_fast = 1'b1;
			 nextstate = IDLE;
		      end
		 end
	       else
		 nextstate = IDLE;
	    end
	  RDATA:
	    begin
	       if (ss_n_sync == 1'b0)
		 begin
		    fsm_spi_load = 1'b1;
		    nextstate = RXDATA;
		 end
	       else
		 nextstate = IDLE;
	    end
	  RXDATA:
	    begin
	       if (ss_n_sync == 1'b0)
		 begin
		    //wait until all data are sendt and then move to IDLE
		    if(spi_fsm_load == 1'b1)
		      begin
			 fsm_spi_load = 1'b1;
			 fsm_spi_load_inc = 1'b1;
		      end
		    if(fsm_data_finish == 1'b1)
		      nextstate = IDLE;
		 end
	       else
		 nextstate = IDLE;
	    end
	  WDATA:
	    begin
	       if (ss_n_sync == 1'b0)
		 begin
		    //wait until all data are received and then mov
		    //the instruction   
		    if (spi_fsm_load == 1'b1)
		      fsm_databuf_load = 1'b1;
		    if (fsm_data_finish == 1'b1)
		      begin
			 fsm_instrreg_load = 1'b1;
			 nextstate = IDLE;
		      end
		 end // if (ss_n_sync == 1'b0)
	       else
		 nextstate = IDLE;
	    end 
	  default:
	    begin
	       nextstate = IDLE;
	    end
	endcase // case(currentstate)
	
     end // always @ (...
   
   
endmodule // spi_co_proc_fsm
