//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  spi_co_proc_if.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <spi_co_proc_if>
//  Description     :  <interface between the alu and the spi>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************

module spi_co_proc_if(/*AUTOARG*/
   // Outputs
   sdo, spi_alu_data, spi_instruction, spi_instrreg_load, debug, 
   // Inputs
   rst_n, clk, sdi, ss_n, sclk, alu_spi_data, alu_spi_write_en
   );

   `include "spi_co_processor_params.v"
   
   //outputs
   output                 sdo;
   output [CPUBITS-1:0]   spi_alu_data;
   output [INSTRBITS-1:0] spi_instruction;
   output 		  spi_instrreg_load;
   output [2:0] 	  debug;
   
   
   //inputs
   input 		rst_n;
   input 		clk;
   input 		sdi;
   input 		ss_n;   
   input 		sclk;
   input [CPUBITS-1:0] 	alu_spi_data;
   input 		alu_spi_write_en;
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire			fsm_databuf_load;	// From U_SPI_CO_PROC_FSM of spi_co_proc_fsm.v
   wire			fsm_instrbuf_lsb_load;	// From U_SPI_CO_PROC_FSM of spi_co_proc_fsm.v
   wire			fsm_instrbuf_msb_load;	// From U_SPI_CO_PROC_FSM of spi_co_proc_fsm.v
   wire			fsm_instrreg_load;	// From U_SPI_CO_PROC_FSM of spi_co_proc_fsm.v
   wire			fsm_instrreg_load_fast;	// From U_SPI_CO_PROC_FSM of spi_co_proc_fsm.v
   wire			fsm_spi_load;		// From U_SPI_CO_PROC_FSM of spi_co_proc_fsm.v
   wire			fsm_spi_load_inc;	// From U_SPI_CO_PROC_FSM of spi_co_proc_fsm.v
   wire [SPIBITS-1:0]	spi_data;		// From U_SPISLAVE of spislave.v
   wire			spi_fsm_load;		// From U_SPISLAVE of spislave.v
   wire			ss_n_sync;		// From U_SPISLAVE of spislave.v
   // End of automatics

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   reg [INSTRBITS-1:0]	spi_instruction;
   // End of automatics

   //other regs and wires
   reg [INSTRBITS-1:0] 	instrbuf;        //instruction register buffer 	
		
   reg [CPUBITS-1:0] 	databufin;       //input data buffer
   reg [CPUBITS-1:0] 	databufout;      //output data buffer
   
   reg [DATAPBITS-1:0] 	datacounter;     //counts spi_data packages 
   reg 			datacounterlast; //the last value of the counter
   reg 			fsm_data_finish; //indicates when the last package is sent

   reg [SPIBITS-1:0] 	data_spi;          //input to spi from databufout
   wire 		spi_rst_n;
   
   assign 		spi_rst_n = rst_n; //assign reset signal
   assign 		spi_instrreg_load = fsm_instrreg_load | fsm_instrreg_load_fast; 
   assign 		spi_alu_data = databufin; 

   /*
    * instruction output 
    * 
    */
   always @ (/*AUTOSENSE*/INSTRBITS or SPIBITS or fsm_instrreg_load
	     or fsm_instrreg_load_fast or instrbuf or spi_data)
     begin
 	if (fsm_instrreg_load == 1'b1)
	  spi_instruction = instrbuf;
	else
	  if (fsm_instrreg_load_fast == 1'b1)
	    spi_instruction = {instrbuf[INSTRBITS-1:INSTRBITS-SPIBITS],spi_data};
	  else
	    spi_instruction = {{INSTRBITS}{1'b0}};
     end
   
		
   /*
    * Instruction input buffer
    * used to sore different 8 bits packages and 
    * put them together
    */
   always @ (posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  instrbuf <= {{INSTRBITS} {1'b0}};
	else
	  begin
	     if (fsm_instrbuf_msb_load == 1'b1)
	       instrbuf <= {spi_data,instrbuf[INSTRBITS-SPIBITS-1:0]};
	     else
	       if (fsm_instrbuf_lsb_load == 1'b1)
		 instrbuf <= {instrbuf[INSTRBITS-1:INSTRBITS-SPIBITS],spi_data};
	  end
     end // always @ (posedge clk)

   /*
    * SPI output data buffer
    *
    */
   always @ (posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  databufout <= {{CPUBITS}{1'b0}};
	else
	  begin
	     if (alu_spi_write_en == 1'b1)
	       databufout <= {alu_spi_data[CPUBITS-SPIBITS-1:0],8'h00};
	     else
	       if (fsm_spi_load == 1'b1)
		 databufout <= {databufout[CPUBITS-SPIBITS-1:0],8'h00};
	  end
     end // always @ (posedge clk)

   always @ (/*AUTOSENSE*/CPUBITS or SPIBITS or alu_spi_data
	     or alu_spi_write_en or databufout or fsm_spi_load)
     begin
	if (fsm_spi_load == 1'b1 && alu_spi_write_en == 1'b1)
	  data_spi = alu_spi_data[CPUBITS-1:CPUBITS-SPIBITS];
	else
	  data_spi = databufout[CPUBITS-1:CPUBITS-SPIBITS];
     end
   
   
      
   /*
    * Data input buffer
    * used to sore different 8 bits packages and 
    * put them together
    */
   always @ (posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  databufin <= {{CPUBITS}{1'b0}};
	else
	  begin
	     if (fsm_databuf_load == 1'b1)
	       databufin <= {databufin[CPUBITS-1-SPIBITS:0],spi_data};
	  end // else: !if(rst_n == 1'b0)
     end // always @ (posedge clk)


   /*
    * Data counter
    * 
    */
   always @ (posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  begin
	     datacounter <= {{DATAPBITS} {1'b1}};
	     datacounterlast <= 1'b0;
	  end
	else
	  begin
	    if((fsm_databuf_load == 1'b1 || fsm_spi_load_inc == 1'b1)&& ss_n_sync == 1'b0)
	      begin
		if (datacounter < DATA_PACKAGES)
		  datacounter <= datacounter + 1;
		else
		  datacounter <= {{DATAPBITS} {1'b0}};
	      end
            else
	      if (ss_n_sync == 1'b1)
	        datacounter <= {{DATAPBITS} {1'b1}};
	    if (datacounter == {{DATAPBITS} {1'b1}})
	      datacounterlast <= 1'b1;
	    else
	      datacounterlast <= 1'b0;
	  end                   // else: !if(rst_n == 1'b0)
     end // always @ (posedge clk)
   
   always @ (/*AUTOSENSE*/DATAPBITS or datacounter or datacounterlast)
     begin
	if (datacounterlast == 1'b0 && datacounter == {{DATAPBITS}{1'b1}})
	  fsm_data_finish = 1'b1;
	else
	  fsm_data_finish = 1'b0;
     end

   
   //finite state machine
   spi_co_proc_fsm U_SPI_CO_PROC_FSM(/*AUTOINST*/
				     // Outputs
				     .fsm_spi_load(fsm_spi_load),
				     .fsm_spi_load_inc(fsm_spi_load_inc),
				     .fsm_instrbuf_msb_load(fsm_instrbuf_msb_load),
				     .fsm_instrbuf_lsb_load(fsm_instrbuf_lsb_load),
				     .fsm_databuf_load(fsm_databuf_load),
				     .fsm_instrreg_load(fsm_instrreg_load),
				     .fsm_instrreg_load_fast(fsm_instrreg_load_fast),
				     .debug(debug[2:0]),
				     // Inputs
				     .clk(clk),
				     .rst_n(rst_n),
				     .spi_fsm_load(spi_fsm_load),
				     .ss_n_sync(ss_n_sync),
				     .instrbuf(instrbuf[INSTRBITS-1:0]),
				     .spi_data(spi_data[SPIBITS-1:0]),
				     .fsm_data_finish(fsm_data_finish));

   //spi slave
   spislave U_SPISLAVE(/*AUTOINST*/
		       // Outputs
		       .sdo		(sdo),
		       .spi_data	(spi_data[SPIBITS-1:0]),
		       .spi_fsm_load	(spi_fsm_load),
		       .ss_n_sync	(ss_n_sync),
		       // Inputs
		       .sclk		(sclk),
		       .clk		(clk),
		       .spi_rst_n	(spi_rst_n),
		       .sdi		(sdi),
		       .ss_n		(ss_n),
		       .data_spi	(data_spi[SPIBITS-1:0]),
		       .fsm_spi_load	(fsm_spi_load));


endmodule // spi_co_proc_if

/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../include"
 "../../rtl/module/trunk/custom/verilog/")
 End:
*/
