//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  spi_co_processor.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <spi_co_processor>
//  Description     :  <alu with spi interface>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module spi_co_processor (/*AUTOARG*/
   // Outputs
   sdo, debug, 
   // Inputs
   rst_n, clk, sdi, ss_n, sclk
   );

   `include "spi_co_processor_params.v"
   
   //outputs
   output sdo;
   output [2:0] debug;
   
   //inputs
   input  rst_n;
   input  clk;
   input  sdi;
   input  ss_n;
   input  sclk;

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   // End of automatics

   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire [OPCODEBITS-1:0]alu_opcode;		// From U_INSTRUCTIONFORMATDECODER of instructionformatdecoder.v
   wire [CPUBITS-1:0]	alu_regfile_data;	// From U_ALU of alu.v
   wire			alu_regfile_write_en;	// From U_ALU of alu.v
   wire [CPUBITS-1:0]	alu_spi_data;		// From U_ALU of alu.v
   wire			alu_spi_write_en;	// From U_ALU of alu.v
   wire [STATUSREGBITS-1:0]alu_statusreg;	// From U_ALU of alu.v
   wire [ADDRBITS-1:0]	regfile_addr_a;		// From U_INSTRUCTIONFORMATDECODER of instructionformatdecoder.v
   wire [ADDRBITS-1:0]	regfile_addr_b;		// From U_INSTRUCTIONFORMATDECODER of instructionformatdecoder.v
   wire [ADDRBITS-1:0]	regfile_addr_d;		// From U_INSTRUCTIONFORMATDECODER of instructionformatdecoder.v
   wire [CPUBITS-1:0]	regfile_alu_a;		// From U_REGISTERFILE of registerfile.v
   wire [CPUBITS-1:0]	regfile_alu_b;		// From U_REGISTERFILE of registerfile.v
   wire [CPUBITS-1:0]	regfile_alu_d;		// From U_REGISTERFILE of registerfile.v
   wire [CPUBITS-1:0]	spi_alu_data;		// From U_SPI_CO_PROC_IF of spi_co_proc_if.v
   wire			spi_instrreg_load;	// From U_SPI_CO_PROC_IF of spi_co_proc_if.v
   wire [INSTRBITS-1:0]	spi_instruction;	// From U_SPI_CO_PROC_IF of spi_co_proc_if.v
   // End of automatics

   //this modules outputs
   wire 		sdo;
   //this modules inputs
   wire 		rst_n;
   wire 		clk;
   wire 		sdi;
   wire 		ss_n;
   wire 		sclk;
   
   //other registers/wires
   reg [STATUSREGBITS-1:0] statusreg_alu;
   reg [INSTRBITS-1:0] 	   instructionreg;
   reg [INSTRBITS-1:0] 	   instructionreg_p1;
 	   
   
   /*
    * status register
    * 
    */
   always @(posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  statusreg_alu <= {{STATUSREGBITS}{1'b0}};
	else
	  statusreg_alu <= alu_statusreg;
     end

      
   /*
    * instruction register ; remove _p1 registers will result in low FC in delay test
    *
    */
   always @ (posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  begin
	     instructionreg_p1 <= {INSTRBITS{1'b0}};
	     instructionreg   <= {INSTRBITS{1'b0}};
	  end
	else
	  begin
	     if (spi_instrreg_load == 1'b1)
               instructionreg_p1 <= spi_instruction; //_p1
	     else
               instructionreg_p1 <= {{INSTRBITS}{1'b0}}; //_p1 <=
	     instructionreg <= instructionreg_p1;
	  end
     end
   //assign instructionreg = spi_instruction;
   

   //spi_co_proc_if
   spi_co_proc_if U_SPI_CO_PROC_IF(/*AUTOINST*/
				   // Outputs
				   .sdo	(sdo),
				   .spi_alu_data(spi_alu_data[CPUBITS-1:0]),
				   .spi_instruction(spi_instruction[INSTRBITS-1:0]),
				   .spi_instrreg_load(spi_instrreg_load),
				   .debug(debug[2:0]),
				   // Inputs
				   .rst_n(rst_n),
				   .clk	(clk),
				   .sdi	(sdi),
				   .ss_n(ss_n),
				   .sclk(sclk),
				   .alu_spi_data(alu_spi_data[CPUBITS-1:0]),
				   .alu_spi_write_en(alu_spi_write_en));
   
   //register file
   registerfile U_REGISTERFILE(/*AUTOINST*/
			       // Outputs
			       .regfile_alu_a(regfile_alu_a[CPUBITS-1:0]),
			       .regfile_alu_b(regfile_alu_b[CPUBITS-1:0]),
			       .regfile_alu_d(regfile_alu_d[CPUBITS-1:0]),
			       // Inputs
			       .clk	(clk),
			       .rst_n	(rst_n),
			       .alu_regfile_write_en(alu_regfile_write_en),
			       .regfile_addr_a(regfile_addr_a[REGFILEADDRBITS-1:0]),
			       .regfile_addr_b(regfile_addr_b[REGFILEADDRBITS-1:0]),
			       .regfile_addr_d(regfile_addr_d[REGFILEADDRBITS-1:0]),
			       .alu_regfile_data(alu_regfile_data[CPUBITS-1:0]));
   
   
   //ALU
   alu U_ALU(/*AUTOINST*/
	     // Outputs
	     .alu_regfile_data		(alu_regfile_data[CPUBITS-1:0]),
	     .alu_spi_data		(alu_spi_data[CPUBITS-1:0]),
	     .alu_statusreg		(alu_statusreg[STATUSREGBITS-1:0]),
	     .alu_regfile_write_en	(alu_regfile_write_en),
	     .alu_spi_write_en		(alu_spi_write_en),
	     // Inputs
	     .regfile_alu_a		(regfile_alu_a[CPUBITS-1:0]),
	     .regfile_alu_b		(regfile_alu_b[CPUBITS-1:0]),
	     .regfile_alu_d		(regfile_alu_d[CPUBITS-1:0]),
	     .spi_alu_data		(spi_alu_data[CPUBITS-1:0]),
	     .alu_opcode		(alu_opcode[OPCODEBITS-1:0]),
	     .statusreg_alu		(statusreg_alu[STATUSREGBITS-1:0]));

   
   //instruction decoder
   instructionformatdecoder U_INSTRUCTIONFORMATDECODER(/*AUTOINST*/
						       // Outputs
						       .alu_opcode(alu_opcode[OPCODEBITS-1:0]),
						       .regfile_addr_a(regfile_addr_a[ADDRBITS-1:0]),
						       .regfile_addr_b(regfile_addr_b[ADDRBITS-1:0]),
						       .regfile_addr_d(regfile_addr_d[ADDRBITS-1:0]),
						       // Inputs
						       .instructionreg(instructionreg[INSTRBITS-1:0]));
   

endmodule // spi_co_processor


/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../include"
 "../../rtl/module/trunk/custom/verilog/")
 End:
*/
   
